
/****************************************************************************/
/*      TCFG2INI v3.00 for Microsoft Windows NT. ELCUS, 1999, 2006.         */
/*      Converts the DOS-style 'tmk.cfg' file to the 'tmk.ini' file         */
/*      which can be used with the REGINI utility.                          */
/****************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <string.h>

int tmkError = 0;

#define MAX_TMK_NUMBER 7

#define MIN_TMK_TYPE 2
#define MAX_TMK_TYPE 10

#define TMK400 2
#define TMKMPC 3
#define RTMK400 4
#define TMKX 5
#define TMKXI 6
#define MRTX 7
#define MRTXI 8
#define TA 9
#define TAI 10

#define TMK_CONFIGURATION_TABLE

#ifdef TMK_CONFIGURATION_TABLE
struct TTmkConfig
{
 int nType;
 char achName[10];
 unsigned wPorts1;
 unsigned wPorts2;
 unsigned wIrq1;
 unsigned wIrq2;
 int nLoadType;
} aTmkConfig[MAX_TMK_NUMBER+1];
#endif

#define TMK_FILE_OPEN_ERROR 21
#define TMK_FILE_READ_ERROR 22
#define TMK_FILE_FORMAT_ERROR 23
#define TMK_UNKNOWN_TYPE 24

unsigned int SGetHexAddr(char **ppszCurParam)
{
 unsigned int Result;
 char *pParEnd;

 while (**ppszCurParam == ' ' || **ppszCurParam == '\t')
  *ppszCurParam++;
 for (pParEnd = *ppszCurParam;
       *pParEnd != ' ' && *pParEnd != '\t' && *pParEnd != '\0'
       && *pParEnd != '\n';
       pParEnd++);
 if (sscanf(*ppszCurParam, "%x", &Result) == 1)
 {
  *ppszCurParam = pParEnd;
  if (*pParEnd != '\0')
  {
   *pParEnd = '\0';
   (*ppszCurParam)++;
  }
 }
 else
  Result = 0;
 return Result;
}

unsigned char SGetIrq(char **ppszCurParam)
{
 unsigned int Result;
 char *pParEnd;

 while (**ppszCurParam == ' ' || **ppszCurParam == '\t')
  *ppszCurParam++;
 for (pParEnd = *ppszCurParam;
       *pParEnd != ' ' && *pParEnd != '\t' && *pParEnd != '\0'
       && *pParEnd != '\n';
       pParEnd++);
 if (*(*ppszCurParam)++ == 'I' &&
      *(*ppszCurParam)++ == 'R' &&
      *(*ppszCurParam)++ == 'Q' &&
      sscanf(*ppszCurParam, "%u", &Result) == 1)
 {
  *ppszCurParam = pParEnd;
  if (*pParEnd != '\0')
  {
   *pParEnd = '\0';
   (*ppszCurParam)++;
  }
 }
 else
  Result = 0;
 return Result;
}

unsigned int SGetLoadType(char **ppszCurParam)
{
 unsigned int Result;
 char *pParEnd;
 char szLoadType[64];

 while (**ppszCurParam == ' ' || **ppszCurParam == '\t')
  *ppszCurParam++;
 for (pParEnd = *ppszCurParam;
       *pParEnd != ' ' && *pParEnd != '\t' && *pParEnd != '\0'
       && *pParEnd != '\n';
       pParEnd++);
 Result = 0;
 if (*(*ppszCurParam)++ == 'L' &&
      sscanf(*ppszCurParam, "%s", &szLoadType) == 1 &&
      (strcmp(szLoadType, "TX1") == 0 ||
        (++Result, strcmp(szLoadType, "RT1") == 0) ||
        (++Result, strcmp(szLoadType, "TX4") == 0) ||
        (++Result, strcmp(szLoadType, "TX5") == 0) ||
        (++Result, strcmp(szLoadType, "TX6") == 0) ||
        (++Result, strcmp(szLoadType, "MR6") == 0) ||
        (++Result, strcmp(szLoadType, "TT6") == 0) ||
        (++Result, strcmp(szLoadType, "TD6") == 0)))
 {
  ++Result;
  *ppszCurParam = pParEnd;
  if (*pParEnd != '\0')
  {
   *pParEnd = '\0';
   (*ppszCurParam)++;
  }
 }
 return Result;
}

int TmkInit(char *pszTMKFileName)
{
 char *apszTmkTypeName[MAX_TMK_TYPE+1] = { "",
                                           "",
                                           "TMK400",
                                           "TMKMPC",
                                           "RTMK400",
                                           "TMKX",
                                           "TMKXI",
                                           "MRTX",
                                           "MRTXI",
                                           "TA",
                                           "TAI" };
 int nResult;
 int hTMK;
 unsigned tmkPorts1, tmkPorts2;
 unsigned char tmkIrq1, tmkIrq2;
 int tmkLoadType;
 char achParams[81];
 char *pCurParam, *pCurParamEnd;
 FILE *hTMKFile;
 unsigned iTMKType;

#ifdef TMK_CONFIGURATION_TABLE
 for (hTMK = 0; hTMK <= MAX_TMK_NUMBER; hTMK++)
 {
  aTmkConfig[hTMK].nType = -1;
  aTmkConfig[hTMK].achName[0] = '\0';
  aTmkConfig[hTMK].wPorts1 = aTmkConfig[hTMK].wPorts2 = 0;
  aTmkConfig[hTMK].wIrq1 = aTmkConfig[hTMK].wIrq2 = 0;
  aTmkConfig[hTMK].nLoadType = 0;
 }
#endif
 if ((hTMKFile = fopen(pszTMKFileName, "r")) == NULL)
  return TMK_FILE_OPEN_ERROR;
 fseek(hTMKFile, 0L, SEEK_SET);
 while (1)
 {
  if (fgets(achParams, 80, hTMKFile) == NULL)
   if (feof(hTMKFile))
    break;
   else
   {
   nResult = TMK_FILE_READ_ERROR;
   goto ExitTmkInit;
   }
  if (achParams[0] == '*')
   break;
  if (sscanf(achParams, "%u", &hTMK) != 1)
   continue;
  if (hTMK > MAX_TMK_NUMBER)
  {
   nResult = TMK_FILE_FORMAT_ERROR;
   goto ExitTmkInit;
  }
  if ((pCurParam = strchr(achParams, ' ')) == NULL)
  {
   nResult = TMK_FILE_FORMAT_ERROR;
   goto ExitTmkInit;
  }
  pCurParam++;
  if ((pCurParamEnd = strchr(pCurParam, ' ')) == NULL)
  {
   nResult = TMK_FILE_FORMAT_ERROR;
   goto ExitTmkInit;
  }
  *pCurParamEnd = '\0';
  for (iTMKType = MIN_TMK_TYPE; iTMKType <= MAX_TMK_TYPE; iTMKType++)
   if (strcmp(apszTmkTypeName[iTMKType], pCurParam) == 0)
    break;
  if (iTMKType > MAX_TMK_TYPE)
  {
   nResult = TMK_UNKNOWN_TYPE;
   goto ExitTmkInit;
  }
  pCurParam = ++pCurParamEnd;
  tmkLoadType = 0;
  switch (iTMKType)
  {
/*
  case RTMK:
  case RTMK1:
*/
  case TMK400:
   if ((tmkPorts1 = SGetHexAddr(&pCurParam)) == 0)
   {
    nResult = TMK_FILE_FORMAT_ERROR;
    goto ExitTmkInit;
   }
   if ((tmkPorts2 = SGetHexAddr(&pCurParam)) == 0)
   {
    nResult = TMK_FILE_FORMAT_ERROR;
    goto ExitTmkInit;
   }
   if ((tmkIrq1 = SGetIrq(&pCurParam)) == 0)
   {
    nResult = TMK_FILE_FORMAT_ERROR;
    goto ExitTmkInit;
   }
   tmkIrq2 = 0xFF;
   break;
  case TMKMPC:
  case RTMK400:
   if ((tmkPorts1 = SGetHexAddr(&pCurParam)) == 0)
   {
    nResult = TMK_FILE_FORMAT_ERROR;
    goto ExitTmkInit;
   }
   if ((tmkIrq1 = SGetIrq(&pCurParam)) == 0)
   {
    nResult = TMK_FILE_FORMAT_ERROR;
    goto ExitTmkInit;
   }
   tmkPorts2 = 0xFFFF;
   tmkIrq2 = 0xFF;
   break;
  case MRTX:
   tmkLoadType = 6;
   iTMKType = TMKX;
  case TMKX:
  case TA:
   if ((tmkPorts1 = SGetHexAddr(&pCurParam)) == 0)
   {
    nResult = TMK_FILE_FORMAT_ERROR;
    goto ExitTmkInit;
   }
   if ((tmkIrq1 = SGetIrq(&pCurParam)) == 0)
   {
    nResult = TMK_FILE_FORMAT_ERROR;
    goto ExitTmkInit;
   }
   if (tmkLoadType == 0)
     tmkLoadType = SGetLoadType(&pCurParam);
   tmkPorts2 = 0xFFFF;
   tmkIrq2 = 0xFF;
   break;
  case MRTXI:
   tmkLoadType = 6;
   iTMKType = TMKXI;
  case TMKXI:
  case TAI:
   if ((tmkPorts1 = SGetHexAddr(&pCurParam)) == 0)
   {
    tmkPorts1 = 1;
   }
   if ((tmkPorts2 = SGetHexAddr(&pCurParam)) == 0)
   {
    tmkPorts2 = 0xFFFF;
   }
   if (tmkLoadType == 0)
     tmkLoadType = SGetLoadType(&pCurParam);
   tmkIrq1 = 0xFF;
   tmkIrq2 = 0xFF;
   break;
  }
#ifdef TMK_CONFIGURATION_TABLE
  aTmkConfig[hTMK].nType = iTMKType;
  strcpy(aTmkConfig[hTMK].achName, apszTmkTypeName[iTMKType]);
  aTmkConfig[hTMK].wPorts1 = tmkPorts1;
  aTmkConfig[hTMK].wPorts2 = tmkPorts2;
  aTmkConfig[hTMK].wIrq1 = tmkIrq1;
  aTmkConfig[hTMK].wIrq2 = tmkIrq2;
  aTmkConfig[hTMK].nLoadType = tmkLoadType;
#endif
  if (tmkError)
   break;
 } /* endwhile(!feof()) */
 nResult = tmkError;
 ExitTmkInit:
 fclose(hTMKFile);
 return nResult;
}

void main()
{
  FILE *fileIni;
  int iTmk;

  if (TmkInit("tmk.cfg"))
  {
    printf("Can't open the configuration file 'tmk.cfg' for read\n");
    goto TmkCfg2Ini_Exit;
  }
  if ((fileIni = fopen("tmk.ini", "wt")) == NULL)
  {
    printf("Can't open the file 'tmk.ini' for write\n");
    goto TmkCfg2Ini_Exit;
  }
  fprintf(
      fileIni,
      "\\Registry\\Machine\\System\\CurrentControlSet\\Services\\Tmk1553b\n"
      "    Parameters [DELETE]\n"
      "    Parameters\n"
      );
  for (iTmk = 0; iTmk <= MAX_TMK_NUMBER; ++iTmk)
  {
    if (aTmkConfig[iTmk].nType == -1)
      continue;
    fprintf(
        fileIni,
        "        %04d\n"
        "            Number = REG_DWORD 0x%08X\n"
        "            Type = REG_DWORD 0x%08X\n"
        "            PortAddress = REG_DWORD 0x%08X\n"
        "            PortAddress2 = REG_DWORD 0x%08X\n"
        "            Interrupt = REG_DWORD 0x%08X\n"
        "            Interrupt2 = REG_DWORD 0x%08X\n",
        iTmk,
        iTmk,
        aTmkConfig[iTmk].nType,
        aTmkConfig[iTmk].wPorts1,
        aTmkConfig[iTmk].wPorts2,
        aTmkConfig[iTmk].wIrq1,
        aTmkConfig[iTmk].wIrq2
        );
    if (aTmkConfig[iTmk].nLoadType)
      fprintf(
          fileIni,
          "            Load = REG_DWORD 0x%08X\n",
          aTmkConfig[iTmk].nLoadType - 1
          ); 
  }
  fclose(fileIni);
  TmkCfg2Ini_Exit:
  return;
}
